//üäöß
#ifndef SHX_HELPER_H
#define SHX_HELPER_H
#include <QtCore>
QStringList shxToolTips =(QStringList()<< 
"TITL [ ]  Title of up to 76 characters"<<
"CELL wavelength[A] a[A] b[A] c[A] alpha[°] beta[°] gamma[°]"<<
"ZERR Z esd(a) esd(b) esd(c) esd(alpha) esd(beta) esd(gamma)"<<
"LATT N  N= 1=>P, 2=>I, 3=>R, 4=>F, 5=>A, 6=>B, 7=>C. Negative N=> non-centrosymmetric"<<
"SYMM symmetry operation"<<
"SFAC elements or \nSFAC label a1 b1 a2 b2 a3 b3 a4 b4 c f' f'' mu r wt"<<
"DISP E f' f'' [#] mu [#]"<<
"UNIT n1 n2 ... \nNumber of atoms of each type in the unit-cell"<<
"LAUE E (values of f' and f\")"<<
"REM the rest of the line is a comment"<<
"MORE m[1] ;Sets the verbosity m=0..3\nNegative m writes a covariance .mat file."<<
"TIME t[s]\nHas been deprecated!"<<
"HOPE\nHas been deprecated!"<<
"END end of input"<<
"HKLF N[0] S[1] r11...r33[1 0 0  0 1 0  0 0 1] sm[1] m[0]"<<
"OMIT s[-2] 2Theta(lim)[180] or\nOMIT h k l\nOMIT atomnames e.g. $H"<<
"SHEL lowres[infinite] highres[0]"<<
"BASF scale factors"<<
"TWIN 3x3 matrix [-1 0 0 0 -1 0 0 0 -1] n[2]"<<
"EXTI x[0]"<<
"SWAT g[0] U[2]"<<
"MERG n[2]"<<
"SPEC del[0.2]"<<
"RESI class[ ] number[0] alias"<<
"MOVE dx[0] dy[0] dz[0] sign[1]"<<
"ANIS n\nANIS names"<<
"AFIX mn d[#] sof[11] U[10.08]"<<
"HFIX mn U[#] d[#] atomnames"<<
"FRAG code[17] a[1] b[1] c[1] alpha[90] beta[90] gamma[90]"<<
"FEND \nThis must immediately follow the last atom of a FRAG fragment."<<
"EXYZ atomnames"<<
"EADP atomnames"<<
"EQIV $n symmetry operation"<<
"CONN bmax[12] r[#] atomnames or \nCONN bmax[12]"<<
"BIND atom1 atom2"<<
"FREE atom1 atom2"<<
"DFIX d s[0.02] pairs of atoms"<<
"DANG d s[0.04] pairs of atoms"<<
"BUMP s [0.02]"<<
"SAME s1[0.02] s2[0.04] atomnames"<<
"SADI s[0.02] pairs of atoms"<<
"CHIV V[0] s[0.1] atomnames"<<
"FLAT s[0.1] four or more atoms"<<
"DELU s1[0.01] s2[0.01] atomnames"<<
"DEFS sd[0.02] sf[0.1] su[0.01] ss[0.04] maxsof[1]"<<
"SIMU s[0.04] st[0.08] dmax[2.0] atomnames"<<
"ISOR s[0.1] st[0.2] atomnames"<<
"NCSY DN sd[0.1] su[0.05] atoms"<<
"SUMP c sigma c1 m1 c2 m2 ..."<<
"L.S. nls[0] nrf[0] nextra[0]"<<
"CGLS nls[0] nrf[0] nextra[0]"<<
"BLOC n1 n2 atomnames"<<
"DAMP damp[0.7] limse[15]"<<
"STIR sres step[0.01]"<<
"WGHT a[0.1] b[0] c[0] d[0] e[0] f[.33333]"<<
"FVAR osf[1] free variables"<<
"BOND atomnames"<<
"CONF atomnames max_d[1.9] max_a[170]"<<
"MPLA na atomnames"<<
"RTAB codename atomnames"<<
"HTAB dh[2.0]\nHTAB donor-atom acceptor-atom"<<
"LIST m[#] mult[1]\nWrite reflection lists to the .fcf file."<<
"ACTA 2thetafull[#]"<<
"SIZE dx dy dz"<<
"TEMP T[20]\nTemperature T of the data collection in degrees Celsius."<<
"WPDB n[1]\nWrites coordinates to a .pdb file."<<
"FMAP code[2] axis[#] nl[53]"<<
"GRID sl[#] sa[#] sd[#] dl[#] da[#] dd[#]"<<
"PLAN npeaks[20] d1[#] d2[#]"<<
"MOLE n\nHas been deprecated!"<<
"XNPD Umin[­0.001]\nrestrain against non-positive-definite atoms"<<
"REST"<<
"CHAN chain id for pdb "<<
"RIGU s1[0.004] s2[0.004] atomnames\nrigid body restraint"<<
"FLAP"<<
"RNUM N offset n to add to all following resi numbers"<<
"SOCC X[11.0] reset occupancy to X for the following atoms"<<
"PRIG p[#]\nSets the parameter p for RIGU"<<
"WIGL del[0.5]\nDisplaces all atoms by distance del"<<
"RANG angle restraints"<<
"TANG torsion angle restraints"<<
"ADDA O2T_999 etc. add atoms to residue so that H-restrains are obeyed."<<
"STAG torsion angle restraint for staggered"<<
"NEUT convert x-ray SFAC table to neutrons"<<
"ABIN n1 n2; Reads name.fab with precalculated structure factors."<<
"ANSC six coefficients\nanisotropic scaling"<<
"ANSR [0.001] restraint for ANSC "<<
"NOTR p[170] d[1.9] removes torsion angles generated by CONF from the .cif file"<<
"TWST N[0] specify twin component used for completeness and Friedel statistics"<<
"BEDE atom1 atom2 r A U1 U2"<<
"LONE atomnames m A U1 U2 r phi\nphi required for m = 2, 3, 7, 9, and 12"//<<
);



#endif // SHX_HELPER_H
