#ifndef ITSME_H
#define ITSME_H
//PROGRAM_NAME is used for settings and saves directory name 
#ifndef PROGRAM_NAME
#define PROGRAM_NAME "shelXle"
#endif
#include <QString>
class ShelXleName{
public:
    QString getName(){return QString(PROGRAM_NAME);}
};
#endif
