//#include "window.h"
// <img src=\":/icons/.png\">
quickReLoadButton->setWhatsThis(
        "<h1>Quick-Re-Load</h1><p><img width=\"128\" src=\":/icons/quick-re-load.png\">"
        "<p> If <b> Quick-Re-Load </b> is checked the structure gets just updated instead of fully fresh reloaded. "
        "This is for example the case when you save the structure or load the results of a refinement. "
        "If <b> Quick-Re-Load </b> is checked then applied symmetry operations and hide-states remain unchanged. ");
wghtRefineAct->setWhatsThis(
    "<h1>Try to refine until WGHT converges (max 10 runs)</h1><p><img src=\":/icons/wghtref.png\">"
    " <p> <b>shelxl</b> recommends a new <b>WGHT</b>ing scheme after every run. This function uses "
    " the recommended scheme and starts a refinement and repeats until the scheme converges, or if "
    " it does not, after a maximum of ten attempts.");

chgl->hideBeLo->setWhatsThis(
    "<h1>Toggle BEDE and LONE objects (IDEAL)</h1><p><img src=\":/icons/ideal.png\"> "
    "<p> The IDEAL (<b>I</b>nvariom <b>D</b>erived <b>E</b>lectron <b>A</b>na<b>L</b>ysis) feature describes the asphericity "
    "of the atomic electron density by adding small Gaussian distributions to chemical bonds and to lone pairs while the same "
    "amount is subtracted at the atom core. At the position of such a Gaussian distribution SheXle draws a triacontahedron. "
    "<p> By clicking on <b>'Toggle BEDE and LONE objects (IDEAL)'</b> you can hide these triacontahedra.");

chgl->clearSelection->setWhatsThis(
    "<h1>Deselect</h1><p><img src=\":/icons/deselect.png\"><p> Deselect all atoms.");

chgl->centroid->setWhatsThis(
    "<h1>Create centroid</h1><p><img src=\":/icons/centroid.png\">"
    "<p>A dummy atom with no contribution to the diffraction pattern is generated at the geometrical center of the selected atoms. ");

chgl->killBond->setWhatsThis(
    "<h1>Free selected atoms</h1><p><img src=\":/icons/free.png\"> <p>This removes the bond between the two selected atoms.");

chgl->addBond->setWhatsThis(
    "<h1>Bind selected atoms</h1><p><img src=\":/icons/bind.png\"> <p>This adds a bond between the two selected atoms.");

chgl->centerSelection->setWhatsThis(
    "<h1>Center selected atoms</h1><p><img src=\":/icons/center.png\"> "
    "<p>Moves the center of rotation to the center of the selected atom(s)");

chgl->invSelection->setWhatsThis(
    "<h1>Invert selection</h1><p><img src=\":/icons/invsel.png\">"
    "<p>Selected atoms become deselected and not selected atoms become deselected.");

chgl->hideNotSelection->setWhatsThis(
    "<h1>Hide other atoms</h1><p><img src=\":/icons/hide.png\">"
    "<p>Everything which is not selected becomes invisible. This might be especially handy if you use it in combination with the"
    "<b>PART<b> and <b>RESI<b>due selectors of the editor, letting you scroll through all your residues or dissordered parts.");

chgl->delSelAt->setWhatsThis(
    "<h1>Delete selected atoms</h1><p><img src=\":/icons/killselected.png\"><p> This deletes selected atoms."
    " <p>You can select atoms by left clicking on them."
    " <p>By pressing <b>[CTRL]</b> you can select multiple atoms."
    " <p>Using <b>[SHIFT]</b> selects all atoms between the already selected atom to the atom clicked on, in the order of the .res-file."
    " <p>Using <b>[SHIFT+CTRL]</b> lets you draw a rectangle within all atoms become selected."
    " <p>You can also use the PART selector and the RESIdue selector of the editor.");

editor->resiFinder->setWhatsThis(
    "<h1>Select residues</h1><p> Select residues defined using <b>RESI</b> instruction. If you click on <img src=\":/icons/hide.png\">"
    "only the selected atoms are shown and you can inspect <b>RESI</b>dues individually. Just scroll on this slider.");

selectPart->setWhatsThis(
    "<h1>Select parts</h1><p> Enable the <b>PART</b> selection mode. Disordered <b>PART</b> can be addressed by their <b>PART</b> number. "
    "The 'incl. Pt 0' options allows for the additional selection of none dissordered <b>PART</b> 0 components together with the selected "
    "<b>PART</b>. If you click on <img src=\":/icons/hide.png\"> only the selected atoms are shown and you can inspect <b>PART</b>s "
    "individually. Just scroll on the spin box next to this.");

editor->setWhatsThis(
    "<h1>.ins/.res editor</h1><p> Text editor to view and manipulate .ins/.res files for refinement purpose. The editor provides syntax "
    "highlighting and auto completing as well as the location of atoms in the structure viewer");

dock->setWhatsThis(
    "<h1>Information window</h1><p> Reports additional information about selected atoms including site symmetry, "
    "bonding distances, angles.");

qpv->setWhatsThis(
    "<h1>Q-Peak Legend <img src=\":/icons/qlegend.png\"> </h1><p> Displays the color scale for the Q-peaks. Left click on the color bar "
    "restricts visible Q-peaks to values above this threshold. A renewed left click on the black box at the bottom displays all Q-peaks "
    "again. If you hide this all Q-Peaks get colored the same.");

chgl->setWhatsThis(
    "<h1>Structure Viewer</h1><p> Displays the structure in the selected drawing mode. The molecule can be rotated "
    "(hold left mouse button), zoomed (hold right mouse button) and moved (hold middle mouse button). Left click on "
    "atoms moves the cursor in the .ins/.res window to this atom, while additional atom informations are reported in "
    "the information window. Distances and angles are measured by selecting multiple atoms and reported in the information "
    "window. Additional actions/options and settings are accessible by right click on atoms.");

refineAct->setWhatsThis(
    "<h1>Run ShelXL</h1><p><img src=\":/icons/refine.png\"><p> Copy .res to .ins and run ShelXL");

refineActAnis->setWhatsThis(
    "<h1>Run ShelXL including anisotropic refinement</h1><p><img src=\":/icons/refineanis.png\"><p> Copy .res to .ins, "
    "insert ANIS instruction and run ShelXL");

refineActActa->setWhatsThis(
    "<h1>Auto HFIX </h1><p><img src=\":/icons/autohfix96.png\"><p>Add Hydrogen atoms automatically and apply suitable AtomFIX "
    "instructions to them. This feature uses the residual density. In some cases it helps to increase the map precision and "
    "to repeat this after refinements.");

openAct->setWhatsThis(
    "<h1>Open </h1><p><img src=\":/icons/open.png\"><p> Open .ins/.res file.");

screenshotAct->setWhatsThis(
    "<h1>Screenshot</h1> <p><img src=\":/icons/diskette.png\"><p> Save screenshot of this structure");

actionSave->setWhatsThis(
    "<h1>Save </h1><p><img src=\":/icons/filesave.png\"><p>Saves the .res files and synchronizes the structure view with the "
    "editor content.");

fxle->mapcontrol->setWhatsThis(
    "<h1>Open map control </h1><p><img src=\":/icons/mapcontrol.png\"><p> Open the Fo and Fo-Fc map control dialog. "
    "In this menu the settings for Fo and Fo-Fc map calculation and representation can be adjusted.");

astdlg->setWhatsThis(
    "<h1> Atom styles </h1><p><img src=\":/icons/eac.png\"><p> Open the atom styles dialog. In this menu atom colors, atom and bond "
    "drawing as well as radii and hydrogen bond setting can be adjusted. Furthermore individual settings can be imported or exported.");

actionEP->setWhatsThis(
    "<img src=\":/icons/efplus.png\"> Increase editor font size");

actionEM->setWhatsThis(
    "<img src=\":/icons/efminus.png\"> Decrease editor font size");

actionUndo->setWhatsThis(
    "<h1>Undo </h1><p><img src=\":/icons/editundo.png\"><p> revert last action");

actionRedo->setWhatsThis(
    "<h1>Redo </h1><p><img src=\":/icons/editredo.png\"><p>  restore last action");

actionCut->setWhatsThis(
    "<h1>Cut </h1><p><img src=\":/icons/editcut.png\"><p> Cut selected text and save to clipboard");

actionCopy->setWhatsThis(
    "<h1> Copy </h1><p><img src=\":/icons/editcut.png\"><p> Copy selected text to clipboard");

actionPaste->setWhatsThis(
    "<h1> Paste </h1><p><img src=\":/icons/editpaste.png\"><p> Paste clipboard content to selected position");

actionSearch->setWhatsThis(
    "<h1>Search and Replace<img src=\":/icons/search.png\"> F3</h1> Open search and replace dialog below the .ins/.res editor");

killQs->setWhatsThis(
    "<h1>Remove Q-Peaks </h1><p><img src=\":/icons/killqs.png\"><p> Remove all Q-Peaks from .res");

killHs->setWhatsThis(
    "<h1>Remove Hydrogens </h1><p><img src=\":/icons/killhs.png\"><p> Remove all hydrogens from .res");

togAtom->setWhatsThis(
    "<h1>Toggle atoms </h1><p><img src=\":/icons/atoms.png\"><p> Toggle drawing of atoms on/off.");

tubeStyle->setWhatsThis(
    "<h1>Tube model </h1><p><img src=\":/icons/tubes.png\"><p> Switch structure representation to tube mode.");

ballStyle->setWhatsThis(
    "<h1>Ball and stick model </h1><p><img src=\":/icons/ballstick.png\"><p> Switch structure representation to ball and stick mode.");

togElli->setWhatsThis(
    "<h1>Toggle ADPs</h1><p><img src=\":/icons/adps.png\"><p> Toggles atom drawing between isotropic and anisotropic "
    "atomic displacement parameter mode.");

togBond->setWhatsThis(
    "<h1>Toggle bonds</h1><p><img src=\":/icons/bonds.png\"><p> Toggles bond drawing on/off.");

togLabel->setWhatsThis(
    "<h1>Toggle labels</h1><p><img src=\":/icons/labels.png\"><p> Toggle atom labels on/off.");

togUnit->setWhatsThis(
    "<h1>Toggle unit cell</h1><p><img src=\":/icons/unitcell.png\"><p> Toggle unit cell on/off");

togHBond->setWhatsThis(
    "<h1>Toggle H-bonds</h1><p><img src=\":/icons/hbrs.png\"><p> Toggle hydrogen bonds on/off");

chgl->atomLegend->setWhatsThis(
    "<h1>Toggle Atom Legend  </h1><p><img src=\":/icons/legend.png\"><p> An atom type legend is shown on the lower left of the view.");

chgl->toggleDockWidgets->setWhatsThis(
    "<h1>Hide text window</h1><p><img src=\":/icons/hidedockwdg.png\"><p> Hide text windows and maximize structure viewer");

toggleOrthoView->setWhatsThis(
    "<h1>Toggle projection mode</h1><p><img src=\":/icons/ortho.png\"><p> Toggle between orthographic and perspective projection mode");

chgl->grow->setWhatsThis(
    "<h1>Toggle FUSE/GROW <img src=\":/icons/grow.png\"><img src=\":/icons/fuse.png\"> </h1> Toggle structure representation between "
    "asymmetric unit cell content (FUSE) and content completed by symmetry (GROW)");

chgl->fuse->setWhatsThis(
    "<h1>Toggle FUSE/GROW <img src=\":/icons/grow.png\"><img src=\":/icons/fuse.png\"> </h1> Toggle structure representation between "
    "asymmetric unit cell content (FUSE) and content completed by symmetry (GROW)");

chgl->qPeakBonds->setWhatsThis(
    "<h1>Q-Peak bonds</h1><p><img src=\":/icons/qbond.png\"><p> Toggle the drawing of bond to and between Q-Peaks on/off");

chgl->hideh->setWhatsThis(
    "<h1>Hydrogens</h1><p><img src=\":/icons/hideh.png\"><p> Toggle hydrogen atom drawing on/off");

chgl->shpm1->setWhatsThis(
    "<h1>Hide PART -N</h1><p><img src=\":/icons/part-1.png\"><p>Hide PART -N symmetry equivalent as ghost atoms");

chgl->highlightParts->setWhatsThis(
    "<h1>Highlight PARTs</h1><p><img src=\":/icons/parts.png\"><p> Highlight structure fragments in dependence of their PART membership.");

enterRenameMode->setWhatsThis(
    "<h1>Rename mode </h1><p><img src=\":/icons/rename.png\"><p> Toggle rename mode on/off. The rename mode facilitates easy "
    "atom type adjusting and renaming");

chgl->invertMouseZoom->setWhatsThis(
    "<h1>Invert mouse zoom </h1><p><img src=\":/icons/invzoom.png\"><p> Invert the mouse move direction for zoom-in and zoom-out.");

homeMe->setWhatsThis(
    "<h1>Move molecules home </h1><p><img src=\":/icons/home.png\"><p> Moves molecule back to the default center of rotation. You can move the "
    "structure dragging the middle mouse button.");

qGrowPeak->setWhatsThis(
    "<h1>Grow Q-Peaks</h1> If the whole molecule is drawn (GROW), this option applies space group symmetry (GROW) also to the Q-peaks.");

search4duplicates->setWhatsThis(
    "<h1>Search for duplicates</h1> Enable the duplicate name check. Duplicates found are highlighted in the .ins/.res editor "
    "windows as well as in the structure viewer for easy localization and renaming.");

zoomIn->setWhatsThis(
    "<h1> Zoom in </h1><p><img src=\":/icons/zoom-in.png\"><p> zoom in.");

zoomOut->setWhatsThis(
    "<h1> Zoom out </h1><p><img src=\":/icons/zoom-out.png\"><p> zoom out.");

fxle->ownfcf6->setWhatsThis(
    "Usual SHELXL and a 'LIST 6' instruction computes a suitable .fcf file. When you check this ShelXle computes the information from the "
    "structure and the .hkl file. This is slower. But you can change an atom type and hit save and see the difference with out refining.");

shxOutputDock->setWhatsThis(
    "<h1>.lst file viewer</h1> The .lst file contains useful information about the refinement of your structure.");

actionHFix->setWhatsThis(
    "<h1>Auto HFIX </h1><p><img src=\":/icons/autohfix96.png\"><p><br>Add Hydrogen atoms automatically and apply "
    "suitable AtomFIX instructions to them. This feature uses the residual density. In some cases it helps to "
    "increase the map presicion and to repeat this after refinements.");
this->setWhatsThis(
"<h2>Key Bindings:</h2><hr><table>\
<tr><td width=\"150\"><b>CTRL + A</b></td>\
<td width=\"300\"><em>Selects all text.</em></td>\
<td width=\"150\"><b>PageDown</b></td>\
<td width=\"300\"><em>Moves the cursor one page down.</em></td>  </tr>\
<tr><td style=\"background:#eeeeff\"><b>CTRL + C</b></td>\
<td style=\"background:#eeeeff\"> Copy the selected text to the clipboard.</td>\
        <td style=\"background:#eeeeff\"><b>PageUp</b></td>\
        <td style=\"background:#eeeeff\"><em>Moves the cursor one page up.</em></td></tr>\
<tr><td><b>CTRL + F</b></td>\
<td> Find in input or list file.</td>\
        <td><b>RightArrow</b></td>\
        <td><em>Moves the cursor one character to the right.</em></td></tr>\
<tr><td style=\"background:#eeeeff\"><b>CTRL + F5</b></td>\
<td style=\"background:#eeeeff\"> Reload structure fram curent file (unsaved changes will be lost).</td>\
        <td style=\"background:#eeeeff\"><b>UpArrow</b></td>\
        <td style=\"background:#eeeeff\"><em>Moves the cursor one line up.</em></td></tr>\
<tr><td><b>CTRL + I</b></td>\
<td> Compute and visualize BODD in direct space (IDEAL).</td>\
        <td><b>Wheel</b></td>\
        <td>In-Decrease label size.</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>CTRL + M</b></td>\
<td style=\"background:#eeeeff\"> Create a rotation movie.</td>\
        <td style=\"background:#eeeeff\"><b>SHIFT + Insert</b></td>\
        <td style=\"background:#eeeeff\">Pastes the clipboard text into text edit.</td></tr>\
<tr><td><b>CTRL + O</b></td>\
<td> Open file.</td>\
        <td><b>SHIFT + Delete</b></td>\
        <td>Deletes the selected text and copies it to the clipboard.</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>CTRL + Q</b></td>\
<td style=\"background:#eeeeff\"> Close ShelXle. (also <b>ALT + F4</b>)</td>\
<td style=\"background:#eeeeff\"><b>SHIFT + F1</b></td>\
<td style=\"background:#eeeeff\">Enter <b><i>WhatsThis Mode</i></b>; Just click on items to get some short help.</td></tr>\
<tr><td><b>CTRL + R</b></td>\
<td> Recalculate distance matrices.</td>\
<td><b>SHIFT + F3</b></td>\
<td>Find previous.</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>CTRL + S</b></td>\
<td style=\"background:#eeeeff\"> Save file.</td>\
        <td style=\"background:#eeeeff\"><b>SHIFT + Wheel</b></td>\
        <td style=\"background:#eeeeff\"> Change value of Fobserved Fourier map.</td></tr>\
<tr><td><b>CTRL + V</b></td>\
<td >Pastes the clipboard text into text edit.</td>\
<td><b>ALT + A</b></td>\
        <td> Toggle atoms.</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>CTRL + X</b></td>\
<td style=\"background:#eeeeff\"> Deletes the selected text and copies it to the clipboard.</td>\
        <td style=\"background:#eeeeff\"><b>ALT + Arrow-Keys</b></td>\
        <td style=\"background:#eeeeff\"> Rotate.</td></tr>\
<tr><td><b>CTRL + Y</b></td>\
<td> Redoes the last operation.</td>\
        <td><b>ALT + B</b></td>\
        <td> Toggle bonds.</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>CTRL + Z</b></td>\
<td style=\"background:#eeeeff\"> Undoes the last operation.</td>\
        <td style=\"background:#eeeeff\"><b>ALT + C</b></td>\
        <td style=\"background:#eeeeff\"> Toggle unit cell.</td></tr>\
<tr><td><b>CTRL + Delete</b></td>\
<td> Selected atom(s) will be deleted.</td>\
        <td><b>ALT + D</b></td>\
        <td> Open DSR plugin.</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>CTRL + End</b></td>\
<td style=\"background:#eeeeff\"> Moves the cursor to the end of the text.</td>\
        <td style=\"background:#eeeeff\"><b>ALT + E</b></td>\
        <td style=\"background:#eeeeff\"> Toggle ADPs.</td></tr>\
<tr><td><b>CTRL + Home</b></td>\
<td> Moves the cursor to the beginning of the text.</td>\
        <td><b>ALT + F</b></td>\
        <td>FUSE shows asymmetric unit only</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>CTRL + Insert</b></td>\
<td style=\"background:#eeeeff\"> Copy the selected text to the clipboard.</td>\
        <td style=\"background:#eeeeff\"><b>ALT + F2</b></td>\
        <td style=\"background:#eeeeff\"> Insert ANIS and run SHELX.</td></tr>\
<tr><td><b>CTRL + LeftArrow</b></td>\
<td> Moves the cursor one word to the left.</td>\
<td><b>ALT + G</b></td>\
<td>GROWs the structure by symmetry</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>CTRL + RightArrow</b></td>\
<td style=\"background:#eeeeff\"> Moves the cursor one word to the right.</td>\
<td style=\"background:#eeeeff\"><b>ALT + H</b></td>\
<td style=\"background:#eeeeff\"> Toggle H-bonds.</td></tr>\
<tr><td><b>CTRL + Wheel</b></td>\
<td> Change value of difference Fourier map.</td>\
<td><b>ALT + K</b></td>\
<td> Toggle atom legend.</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>CTRL + +</b></td>\
<td style=\"background:#eeeeff\"><em>Zooms text</em></td>\
<td style=\"background:#eeeeff\"><b>ALT + L</b></td>\
<td style=\"background:#eeeeff\"> Toggle labels.</td></tr>\
<tr><td><b>CTRL + -</b></td>\
<td><em>Unzooms text</em></td>\
<td><b>ALT + PageDown</b></td>\
<td> Zoom out.</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>Backspace</b></td>\
<td style=\"background:#eeeeff\">Deletes the character to the left of the cursor.</td>\
<td style=\"background:#eeeeff\"><b>ALT + PageUp</b></td>\
<td style=\"background:#eeeeff\"> Zoom in.</td></tr>\
<tr><td><b>Delete</b></td><td>Deletes the character to the right of the cursor.</td>\
<td><b>ALT + R</b></td>\
<td> Enter rename mode.</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>DownArrow</b></td>\
<td style=\"background:#eeeeff\"><em>Moves the cursor one line down.</em></td>\
<td style=\"background:#eeeeff\"><b>F7</b></td>\
<td style=\"background:#eeeeff\"> Save a screen shot.</td></tr>\
<tr><td><b>End</b></td>\
<td>Moves the cursor to the end of the line <em>(text)</em>.</td>\
<td><b>ALT + Wheel</b></td><td><em>Scrolls the page horizontally.</em></td></tr>\
<tr><td style=\"background:#eeeeff\"><b>F1</b></td>\
<td style=\"background:#eeeeff\"> (With text cursor on a SHELXL command) Opens a web browser of SHELX online manual.</td>\
<td style=\"background:#eeeeff\"><b>ALT + +</b></td>\
<td style=\"background:#eeeeff\"> Increase label font size.</td></tr>\
<tr><td><b>F2</b></td>\
<td> Run SHELX (followed by <b>SPACE</b> after reinement is done to load refinement results)</td>\
<td><b>ALT + -</b></td>\
<td> Decrease label font size.</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>F3</b></td>\
<td style=\"background:#eeeeff\"> Find/Find next in input file.</td>\
<td style=\"background:#eeeeff\"><b>ALT + SHIFT + A</b></td>\
<td style=\"background:#eeeeff\">Change atom styles.</td></tr>\
<tr><td><b>F4</b></td>\
<td> Toggle multi sample anti aliasing MSAA.</td>\
<td><b>ALT + SHIFT + B</b></td>\
<td>Change background color.</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>F5</b></td><td style=\"background:#eeeeff\"> Redraw structure.</td>\
<td style=\"background:#eeeeff\"><b>ALT + SHIFT + L</b></td><td style=\"background:#eeeeff\">Short Labels.</td></tr>\
<tr><td><b>F6</b></td><td> Toggle background gradient.</td>\
</tr>\
<tr><td style=\"background:#eeeeff\"><b>META + SHIFT + H</b></td>\
<td style=\"background:#eeeeff\"> Delete all hydrogen atoms.</td>\
</tr>\
<tr><td><b>META + SHIFT + Q</b></td>\
<td> Delete all q-peaks.</td>\
        <td><b>ALT + SHIFT + T</b></td>\
        <td>Change label color.</td></tr>\
<tr><td style=\"background:#eeeeff\"><b>Home</b></td>\
<td style=\"background:#eeeeff\">Moves the cursor to the beginning of the line<em>(text)</em>.</td>\
        <td style=\"background:#eeeeff\"><b>CTRL + ALT + S</b></td>\
        <td style=\"background:#eeeeff\"> Save as (was not working before 1138).</td></tr>\
<tr><td><b>LeftArrow</b></td>\
<td><em>Moves the cursor one character to the left.</em></td>\
        <td><b>CTRL + SHIFT + Drag</b></td>\
        <td> Select atoms in a rectangular region.</td></tr>\
</table><HR>(<b>META</b> = Windows key); <b>Wheel</b> = Mousewheel");
//<td><hr></td><td><hr></td>
//<td><hr></td><td><hr></td>
//<td><hr></td><td><hr></td>
