#ifndef INCLUDEEDITOR_H
#define INCLUDEEDITOR_H

#include <QObject>
#include <QDialog>
#include "codeeditor.h"

class IncludeEditor : public QDialog
{
    Q_OBJECT
public:
    explicit IncludeEditor(QString inclstr, const QString &_dir, QWidget *parent = NULL  , Qt::WindowFlags f = 0);
    QPushButton * ok;//!> the ok button
    void setDirName(const QString &_dir);
signals:

public slots:
  void accept();
private:
    QString fileName;
    QString dirName;
    CodeEditor *edi;
};

#endif // INCLUDEEDITOR_H
